@@ -462,6 +462,49 @@ pub struct RouteChange {
     pub at_seq: u64,
 }
 
+/// A recorded probe event for replay animation
+#[derive(Debug, Clone, Serialize, Deserialize)]
+pub struct ProbeEvent {
+    /// Milliseconds since session start
+    pub offset_ms: u64,
+    /// TTL of the probe
+    pub ttl: u8,
+    /// Sequence number for probe correlation
+    #[serde(default)]
+    pub seq: u8,
+    /// Flow ID for multi-flow ECMP tracking
+    #[serde(default)]
+    pub flow_id: u8,
+    /// Probe outcome
+    #[serde(flatten)]
+    pub outcome: ProbeOutcome,
+}
+
+/// Outcome of a probe (reply or timeout)
+#[derive(Debug, Clone, Serialize, Deserialize)]
+#[serde(tag = "type")]
+pub enum ProbeOutcome {
+    /// Received a reply from a hop
+    #[serde(rename = "reply")]
+    Reply {
+        /// IP address of the responder
+        addr: IpAddr,
+        /// Round-trip time in microseconds
+        rtt_us: u64,
+    },
+    /// Probe timed out with no response
+    #[serde(rename = "timeout")]
+    Timeout,
+    /// Late reply received after timeout was already recorded
+    #[serde(rename = "late_reply")]
+    LateReply {
+        /// IP address of the responder
+        addr: IpAddr,
+        /// Round-trip time in microseconds
+        rtt_us: u64,
+    },
+}
+
 /// Asymmetric routing detection information for a hop
 ///
 /// Detects when the return path (from router back to us) differs from the
@@ -720,14 +763,16 @@ pub struct PmtudState {
 impl PmtudState {
     /// Create new PMTUD state with appropriate bounds for IPv4 or IPv6
     ///
-    /// - IPv4: min=68 (RFC 791), max=1500
-    /// - IPv6: min=1280 (RFC 8200), max=1500
-    pub fn new(ipv6: bool) -> Self {
+    /// - IPv4: min=68 (RFC 791)
+    /// - IPv6: min=1280 (RFC 8200)
+    /// - max=1500 for standard ethernet, 9216 with --jumbo flag
+    pub fn new(ipv6: bool, jumbo: bool) -> Self {
         let min = if ipv6 { 1280 } else { 68 };
+        let max = if jumbo { 9216 } else { 1500 };
         Self {
             min_size: min,
-            max_size: 1500,
-            current_size: 1500, // Start high, binary search down
+            max_size: max,
+            current_size: max, // Start high, binary search down
             successes: 0,
             failures: 0,
             discovered_mtu: None,
@@ -782,13 +827,23 @@ impl PmtudState {
     }
 
     /// Record ICMP Fragmentation Needed with reported MTU
-    /// Immediately clamps max_size to the reported MTU
+    /// Per RFC 1191, the router explicitly tells us the path MTU
     pub fn record_frag_needed(&mut self, reported_mtu: u16) {
-        // Clamp immediately - no need for multiple confirmations
-        self.max_size = self.max_size.min(reported_mtu);
-        self.successes = 0;
-        self.failures = 0;
-        self.advance();
+        if reported_mtu < self.max_size {
+            // Router is telling us a lower MTU than we knew - trust it directly
+            // This is the path MTU, complete immediately
+            self.discovered_mtu = Some(reported_mtu);
+            self.phase = PmtudPhase::Complete;
+            self.max_size = reported_mtu;
+            self.min_size = self.min_size.min(reported_mtu);
+        } else {
+            // Reported MTU >= our current max (e.g., 9000 jumbo on a 1500 link)
+            // Can't trust it, continue binary search with current bounds
+            self.max_size = self.max_size.min(reported_mtu);
+            self.successes = 0;
+            self.failures = 0;
+            self.advance();
+        }
     }
 
     /// Advance to next probe size or complete if converged
@@ -1168,11 +1223,14 @@ impl Target {
 pub struct Session {
     pub target: Target,
     pub started_at: DateTime<Utc>,
+    /// Monotonic start time for offset calculations (avoids clock jumps)
+    #[serde(skip)]
+    pub started_instant: Option<std::time::Instant>,
     pub hops: Vec<Hop>,
     pub config: Config,
     pub complete: bool,       // destination reached?
     pub dest_ttl: Option<u8>, // TTL at which destination was reached (actual hop count)
-    pub total_sent: u64,      // total probes sent across all hops
+    pub total_sent: u64,      // total completed probes (response or timeout) across all hops
     #[serde(skip)]
     pub paused: bool, // pause probing (TUI only)
     /// PMTUD state (only present when --pmtud is enabled)
@@ -1184,6 +1242,9 @@ pub struct Session {
     /// Default gateway IP (for display in TUI)
     #[serde(default, skip_serializing_if = "Option::is_none")]
     pub gateway: Option<IpAddr>,
+    /// Recorded probe events for animated replay
+    #[serde(default, skip_serializing_if = "Vec::is_empty")]
+    pub events: Vec<ProbeEvent>,
 }
 
 impl Session {
@@ -1196,7 +1257,7 @@ impl Session {
 
         // Initialize PMTUD state if enabled
         let pmtud = if config.pmtud {
-            Some(PmtudState::new(target.resolved.is_ipv6()))
+            Some(PmtudState::new(target.resolved.is_ipv6(), config.jumbo))
         } else {
             None
         };
@@ -1204,6 +1265,7 @@ impl Session {
         Self {
             target,
             started_at: Utc::now(),
+            started_instant: Some(std::time::Instant::now()),
             hops,
             config,
             complete: false,
@@ -1213,6 +1275,7 @@ impl Session {
             pmtud,
             source_ip: None,
             gateway: None,
+            events: Vec::new(),
         }
     }
 
@@ -1252,12 +1315,19 @@ impl Session {
         self.complete = false;
         self.dest_ttl = None;
         self.started_at = Utc::now();
+        self.started_instant = Some(std::time::Instant::now());
 
         // Reset PMTUD state if enabled
         if self.pmtud.is_some() {
-            self.pmtud = Some(PmtudState::new(self.target.resolved.is_ipv6()));
+            self.pmtud = Some(PmtudState::new(
+                self.target.resolved.is_ipv6(),
+                self.config.jumbo,
+            ));
         }
 
+        // Clear recorded events
+        self.events.clear();
+
         for hop in &mut self.hops {
             hop.sent = 0;
             hop.received = 0;
@@ -1285,6 +1355,46 @@ impl Session {
     pub fn first_nat_hop(&self) -> Option<u8> {
         self.hops.iter().find(|h| h.has_nat()).map(|h| h.ttl)
     }
+
+    /// Clone session data for TUI rendering, excluding events (which can be large).
+    /// The events vec is only used for replay export, never during TUI rendering.
+    pub fn snapshot_for_render(&self) -> Self {
+        Self {
+            target: self.target.clone(),
+            started_at: self.started_at,
+            started_instant: self.started_instant,
+            hops: self.hops.clone(),
+            config: self.config.clone(),
+            complete: self.complete,
+            dest_ttl: self.dest_ttl,
+            total_sent: self.total_sent,
+            paused: self.paused,
+            pmtud: self.pmtud.clone(),
+            source_ip: self.source_ip,
+            gateway: self.gateway,
+            events: Vec::new(),
+        }
+    }
+
+    /// Record a probe event for animated replay
+    pub fn record_event(&mut self, event: ProbeEvent) {
+        self.events.push(event);
+    }
+
+    /// Get monotonic offset in milliseconds since session start.
+    /// Uses Instant for monotonic timing (avoids clock jumps).
+    /// Falls back to Utc for deserialized sessions (where started_instant is None).
+    pub fn offset_ms(&self) -> u64 {
+        if let Some(instant) = self.started_instant {
+            instant.elapsed().as_millis() as u64
+        } else {
+            // Fallback for deserialized sessions
+            Utc::now()
+                .signed_duration_since(self.started_at)
+                .num_milliseconds()
+                .unsigned_abs()
+        }
+    }
 }
 
 /// Serde helper for Duration
@@ -1296,7 +1406,7 @@ mod duration_serde {
     where
         S: Serializer,
     {
-        duration.as_micros().serialize(serializer)
+        (duration.as_micros() as u64).serialize(serializer)
     }
 
     pub fn deserialize<'de, D>(deserializer: D) -> Result<Duration, D::Error>
