@@ -62,16 +62,16 @@ impl ProbeEngine {
 
     /// Apply rate limiting delay if configured
     ///
-    /// On macOS, a minimum delay is always applied even without --rate.
-    /// This is required because macOS batches rapid setsockopt(IP_TTL) calls,
+    /// On macOS/FreeBSD, a minimum delay is always applied even without --rate.
+    /// This is required because BSD-derived systems batch rapid setsockopt(IP_TTL) calls,
     /// causing packets to be sent with stale TTL values. A small delay ensures
     /// each set_ttl() takes effect before the next send().
     /// See: https://github.com/lance0/ttl/issues/12
     async fn apply_rate_limit(&self) {
         if let Some(delay) = self.rate_delay() {
             tokio::time::sleep(delay).await;
-        } else if cfg!(target_os = "macos") {
-            // macOS requires a minimum delay between probes to ensure
+        } else if cfg!(any(target_os = "macos", target_os = "freebsd")) {
+            // macOS/FreeBSD require a minimum delay between probes to ensure
             // setsockopt(IP_TTL) takes effect before each send().
             // Without this, rapid probe bursts all get sent with the same TTL.
             // 500µs provides sufficient margin for the kernel to process the sockopt change.
@@ -151,19 +151,20 @@ impl ProbeEngine {
 
         // Bind to source IP if configured OR if IPv6 (required for checksum consistency)
         // Skip binding if source is unspecified (:: or 0.0.0.0) - let kernel choose
-        if (self.config.source_ip.is_some() || ipv6) && !src_ip.is_unspecified() {
-            if let Err(e) = bind_to_source_ip(&socket, src_ip) {
-                if self.config.source_ip.is_some() {
-                    // User explicitly requested this source IP - hard fail
-                    return Err(e);
-                }
-                // Auto-detected source IP failed to bind (e.g., link-local scope mismatch)
-                // Warn and continue - kernel will choose source, checksum may be wrong
-                eprintln!(
-                    "Warning: Failed to bind to source IP {}: {}. IPv6 checksum may be incorrect.",
-                    src_ip, e
-                );
+        if (self.config.source_ip.is_some() || ipv6)
+            && !src_ip.is_unspecified()
+            && let Err(e) = bind_to_source_ip(&socket, src_ip)
+        {
+            if self.config.source_ip.is_some() {
+                // User explicitly requested this source IP - hard fail
+                return Err(e);
             }
+            // Auto-detected source IP failed to bind (e.g., link-local scope mismatch)
+            // Warn and continue - kernel will choose source, checksum may be wrong
+            eprintln!(
+                "Warning: Failed to bind to source IP {}: {}. IPv6 checksum may be incorrect.",
+                src_ip, e
+            );
         }
 
         let mut seq: u8 = 0;
@@ -268,16 +269,6 @@ impl ProbeEngine {
                             continue;
                         }
 
-                        // Record that we sent a probe
-                        {
-                            let mut state = self.state.write();
-                            if let Some(hop) = state.hop_mut(ttl) {
-                                hop.record_sent();
-                                hop.record_flow_sent(0); // ICMP uses single flow (checksum trick not yet implemented)
-                            }
-                            state.total_sent += 1;
-                        }
-
                         // Apply rate limiting if configured
                         self.apply_rate_limit().await;
                     }
@@ -424,16 +415,6 @@ impl ProbeEngine {
                                 continue;
                             }
 
-                            // Record that we sent a probe
-                            {
-                                let mut state = self.state.write();
-                                if let Some(hop) = state.hop_mut(ttl) {
-                                    hop.record_sent();
-                                    hop.record_flow_sent(flow_id);
-                                }
-                                state.total_sent += 1;
-                            }
-
                             // Apply rate limiting if configured
                             self.apply_rate_limit().await;
                         }
@@ -562,16 +543,6 @@ impl ProbeEngine {
                                 continue;
                             }
 
-                            // Record that we sent a probe
-                            {
-                                let mut state = self.state.write();
-                                if let Some(hop) = state.hop_mut(ttl) {
-                                    hop.record_sent();
-                                    hop.record_flow_sent(flow_id);
-                                }
-                                state.total_sent += 1;
-                            }
-
                             // Apply rate limiting if configured
                             self.apply_rate_limit().await;
                         }
@@ -700,10 +671,7 @@ impl ProbeEngine {
         // Send the probe
         match send_icmp(socket, &packet, self.target) {
             Ok(_) => {
-                // Don't increment hop.sent for PMTUD probes - they're for MTU discovery,
-                // not traceroute measurements. Only increment total_sent for diagnostics.
-                let mut state = self.state.write();
-                state.total_sent += 1;
+                // Sent counting deferred to receiver for atomic stat updates
                 true
             }
             Err(e) => {
@@ -754,7 +722,7 @@ impl ProbeEngine {
         // Set socket to non-blocking for polling
         let _ = socket.set_nonblocking(true);
 
-        let mut buffer = [0u8; 1500];
+        let mut buffer = [0u8; 9216];
 
         // Drain any pending Echo Reply responses
         loop {
@@ -796,17 +764,18 @@ impl ProbeEngine {
 
                         // Update state with parity to receiver behavior
                         let mut state = self.state.write();
+                        state.total_sent += 1;
 
                         // Only record hop stats for normal probes, not PMTUD probes
-                        if !is_pmtud_probe {
-                            if let Some(hop) = state.hop_mut(parsed.probe_id.ttl) {
-                                // Use flap-detecting record for single-flow mode (ICMP is always single-flow)
-                                hop.record_response_detecting_flaps(parsed.responder, rtt, None);
-                                hop.record_flow_response(flow_id, parsed.responder, rtt);
-                                // Record response TTL for asymmetry detection
-                                if let Some(response_ttl) = recv_result.response_ttl {
-                                    hop.record_response_ttl(response_ttl, true);
-                                }
+                        if !is_pmtud_probe && let Some(hop) = state.hop_mut(parsed.probe_id.ttl) {
+                            hop.record_sent();
+                            hop.record_flow_sent(flow_id);
+                            // Use flap-detecting record for single-flow mode (ICMP is always single-flow)
+                            hop.record_response_detecting_flaps(parsed.responder, rtt, None);
+                            hop.record_flow_response(flow_id, parsed.responder, rtt);
+                            // Record response TTL for asymmetry detection
+                            if let Some(response_ttl) = recv_result.response_ttl {
+                                hop.record_response_ttl(response_ttl, true);
                             }
                         }
 
@@ -814,20 +783,18 @@ impl ProbeEngine {
                         if parsed.responder == self.target {
                             state.complete = true;
                             let ttl = parsed.probe_id.ttl;
-                            if state.dest_ttl.is_none() || ttl < state.dest_ttl.unwrap() {
+                            if state.dest_ttl.is_none_or(|d| ttl < d) {
                                 state.dest_ttl = Some(ttl);
                             }
                         }
 
                         // Handle PMTUD probe success
-                        if let Some(probe_size) = probe.packet_size {
-                            if let Some(ref mut pmtud) = state.pmtud {
-                                if pmtud.phase == PmtudPhase::Searching
-                                    && probe_size == pmtud.current_size
-                                {
-                                    pmtud.record_success();
-                                }
-                            }
+                        if let Some(probe_size) = probe.packet_size
+                            && let Some(ref mut pmtud) = state.pmtud
+                            && pmtud.phase == PmtudPhase::Searching
+                            && probe_size == pmtud.current_size
+                        {
+                            pmtud.record_success();
                         }
                     }
                 }
