@@ -22,6 +22,7 @@ fn test_session_with_pmtud() -> Session {
     let target = Target::new("8.8.8.8".to_string(), IpAddr::V4(Ipv4Addr::new(8, 8, 8, 8)));
     let config = Config {
         pmtud: true,
+        jumbo: true, // Use jumbo mode to test full 9216 range
         ..Default::default()
     };
     Session::new(target, config)
@@ -35,6 +36,7 @@ fn test_session_ipv6_with_pmtud() -> Session {
     );
     let config = Config {
         pmtud: true,
+        jumbo: true, // Use jumbo mode to test full 9216 range
         ..Default::default()
     };
     Session::new(target, config)
@@ -302,7 +304,7 @@ fn test_pmtud_state_lifecycle() {
     let pmtud = session.pmtud.as_ref().unwrap();
     assert_eq!(pmtud.phase, PmtudPhase::WaitingForDestination);
     assert_eq!(pmtud.min_size, 68); // IPv4 minimum
-    assert_eq!(pmtud.max_size, 1500);
+    assert_eq!(pmtud.max_size, 9216);
     assert_eq!(pmtud.discovered_mtu, None);
 
     // Simulate PMTUD progress
@@ -323,7 +325,7 @@ fn test_pmtud_state_lifecycle() {
     let pmtud = session.pmtud.as_ref().unwrap();
     assert_eq!(pmtud.phase, PmtudPhase::WaitingForDestination);
     assert_eq!(pmtud.min_size, 68);
-    assert_eq!(pmtud.max_size, 1500);
+    assert_eq!(pmtud.max_size, 9216);
     assert_eq!(pmtud.discovered_mtu, None);
 }
 
@@ -366,14 +368,26 @@ fn test_pmtud_ipv6_min_size() {
     assert!(session.pmtud.is_some());
     let pmtud = session.pmtud.as_ref().unwrap();
     assert_eq!(pmtud.min_size, 1280);
-    assert_eq!(pmtud.max_size, 1500);
+    assert_eq!(pmtud.max_size, 9216);
     assert_eq!(pmtud.phase, PmtudPhase::WaitingForDestination);
 }
 
+#[test]
+fn test_pmtud_default_max_is_1500() {
+    // Non-jumbo mode should use 1500 (standard ethernet) as max
+    let pmtud_ipv4 = PmtudState::new(false, false); // IPv4, non-jumbo
+    assert_eq!(pmtud_ipv4.min_size, 68);
+    assert_eq!(pmtud_ipv4.max_size, 1500);
+
+    let pmtud_ipv6 = PmtudState::new(true, false); // IPv6, non-jumbo
+    assert_eq!(pmtud_ipv6.min_size, 1280);
+    assert_eq!(pmtud_ipv6.max_size, 1500);
+}
+
 #[test]
 fn test_pmtud_frag_needed_below_min() {
-    // Test that record_frag_needed clamps to reported MTU even if below typical min
-    let mut pmtud = PmtudState::new(false); // IPv4
+    // Test that record_frag_needed trusts ICMP-reported MTU per RFC 1191
+    let mut pmtud = PmtudState::new(false, false); // IPv4
     pmtud.start_search();
 
     // Router reports MTU of 576 (old internet minimum)
@@ -382,18 +396,15 @@ fn test_pmtud_frag_needed_below_min() {
     // max_size should be clamped to 576
     assert_eq!(pmtud.max_size, 576);
 
-    // min_size stays at 68 (IPv4 absolute minimum)
-    assert_eq!(pmtud.min_size, 68);
-
-    // Binary search should continue in valid range
-    assert!(pmtud.current_size <= 576);
-    assert!(pmtud.current_size >= 68);
+    // Per RFC 1191, router-reported MTU is trusted directly - complete immediately
+    assert_eq!(pmtud.phase, PmtudPhase::Complete);
+    assert_eq!(pmtud.discovered_mtu, Some(576));
 }
 
 #[test]
 fn test_pmtud_frag_needed_at_min() {
     // Edge case: reported MTU equals or is below min_size
-    let mut pmtud = PmtudState::new(true); // IPv6, min=1280
+    let mut pmtud = PmtudState::new(true, false); // IPv6, min=1280
     pmtud.start_search();
 
     // Router reports exactly 1280
@@ -438,7 +449,7 @@ fn test_pmtud_reset_clears_discovered_mtu() {
     let pmtud = session.pmtud.as_ref().unwrap();
     assert_eq!(pmtud.phase, PmtudPhase::WaitingForDestination);
     assert_eq!(pmtud.min_size, 68);
-    assert_eq!(pmtud.max_size, 1500);
+    assert_eq!(pmtud.max_size, 9216);
     assert_eq!(pmtud.discovered_mtu, None); // Key assertion
     assert_eq!(pmtud.successes, 0);
     assert_eq!(pmtud.failures, 0);
@@ -475,21 +486,36 @@ fn test_pmtud_reset_clears_discovered_mtu_ipv6() {
     let pmtud = session.pmtud.as_ref().unwrap();
     assert_eq!(pmtud.phase, PmtudPhase::WaitingForDestination);
     assert_eq!(pmtud.min_size, 1280); // IPv6 minimum
-    assert_eq!(pmtud.max_size, 1500);
+    assert_eq!(pmtud.max_size, 9216);
     assert_eq!(pmtud.discovered_mtu, None);
 }
 
 #[test]
-fn test_pmtud_frag_needed_above_max() {
-    // Edge case: router reports MTU > 1500 (jumbo frames or bogus value)
-    let mut pmtud = PmtudState::new(false); // IPv4
+fn test_pmtud_frag_needed_jumbo() {
+    // Router reports jumbo MTU within our search range
+    let mut pmtud = PmtudState::new(false, true); // IPv4, jumbo mode
     pmtud.start_search();
 
-    // Router reports 9000 (jumbo frame MTU)
+    // Router reports 9000 (jumbo frame MTU) - within our max of 9216
     pmtud.record_frag_needed(9000);
 
-    // max_size should stay at 1500 (min of current max and reported)
-    assert_eq!(pmtud.max_size, 1500);
+    // 9000 < 9216, so router-reported MTU is trusted per RFC 1191
+    assert_eq!(pmtud.max_size, 9000);
+    assert_eq!(pmtud.discovered_mtu, Some(9000));
+    assert_eq!(pmtud.phase, PmtudPhase::Complete);
+}
+
+#[test]
+fn test_pmtud_frag_needed_above_max() {
+    // Edge case: router reports MTU above our ceiling (bogus value)
+    let mut pmtud = PmtudState::new(false, true); // IPv4, jumbo mode
+    pmtud.start_search();
+
+    // Router reports 16000 (above our 9216 ceiling)
+    pmtud.record_frag_needed(16000);
+
+    // max_size should stay at 9216 (min of current max and reported)
+    assert_eq!(pmtud.max_size, 9216);
 
     // Search continues normally
     assert_eq!(pmtud.phase, PmtudPhase::Searching);
@@ -498,7 +524,7 @@ fn test_pmtud_frag_needed_above_max() {
 #[test]
 fn test_pmtud_ipv6_convergence() {
     // Full IPv6 PMTUD cycle with realistic MTU
-    let mut pmtud = PmtudState::new(true); // IPv6, min=1280
+    let mut pmtud = PmtudState::new(true, false); // IPv6, min=1280
     pmtud.start_search();
 
     // Simulate link with 1400 byte MTU (common for tunnels)
@@ -768,10 +794,11 @@ fn test_session_json_file_roundtrip() {
     assert_eq!(loaded.hop(2).unwrap().timeouts, 1);
     assert_eq!(loaded.hop(3).unwrap().primary, Some(target));
 
-    // Verify PMTUD state
+    // Verify PMTUD state - with RFC 1191 behavior, record_frag_needed completes immediately
     assert!(loaded.pmtud.is_some());
     let pmtud = loaded.pmtud.as_ref().unwrap();
-    assert_eq!(pmtud.phase, PmtudPhase::Searching);
+    assert_eq!(pmtud.phase, PmtudPhase::Complete);
+    assert_eq!(pmtud.discovered_mtu, Some(1400));
     assert!(pmtud.max_size <= 1400);
 
     // Cleanup
